///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   AddSubOp.h
//! \brief  Defines the mcFractionData, mcFractionGUI, mcFractionMath,
//!         mcFractionIO, mcFraction classes.
//!
//! See Fraction.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef ADDSUBOP_H
#define ADDSUBOP_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "AddSubOp.h"
#endif

// required includes
#include "mc/Operator.h"


// this will be defined later...
mcDEFINE_HELPER_CLASSES(mcAddSubOp)


//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcAddSubOpHelpers : public mcOperatorHelpers
{
	mcDEFINE_REFERENCE_DATA(mcAddSubOp, mdata_nType);

public:
	mcAddSubOpHelpers() { data_Init(); }
	virtual ~mcAddSubOpHelpers() {}


#ifdef mcENABLE_DATA
public:
	bool data_isAddOp() const		{ return data_GetType() == mcET_ADDOP; }
	bool data_isSubOp() const		{ return data_GetType() == mcET_SUBOP; }


#ifdef __MCDEBUG__
	wxString data_Debug(long flags) const 
		{ return wxString(wxT("mcAddSubOp    ")) + (data_isAddOp() ? wxT("+\n") : wxT("-\n")); }
#endif
#endif		// mcENABLE_DATA


#ifdef mcENABLE_MATH
public:
	bool math_CanBeApplied(const mcElement &, const mcElement &) const;
	mcRealValue math_Evaluate(mcRealValue v1, mcRealValue v2) const;

	mcBasicOpRes math_ApplySimple(mcElement &m1, const mcElement &m2) const;
	mcBasicOpRes math_Apply(mcElement &, const mcElement &, mcElement *) const;

#endif		// mcENABLE_MATH
};


//! An empty class which creates a mcAddSubOpMath section and
//! for GUI, DATA and IO uses mcOperator helper classes.
class mcAddSubOp : public mcOperator
{
	mcDEFINE_MAIN_CLASS(AddSubOp, mcOperator);

public:

	//! With this constructor, you can build a mcAddSubOp
	//! working as + or a mcAddSubOp working as -.
	mcAddSubOp(mcElementType t);
	

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_ADDOP || t == mcET_SUBOP; }

	bool data_isAddOp() const			{ return hlp()->data_isAddOp(); }
	bool data_isSubOp() const			{ return hlp()->data_isSubOp(); }
};

#endif		// ADDSUBOP_H
