///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   Docs.h
//! \brief  A simple text file with various doxygen help pages on 
//!         main MathCore topics.
//!
//! This file contains some special comments which are used by Doxygen
//! to create the hand-written pages. These pages uses doxygen special
//! commands & some HTML commands. You should not read directly this file;
//! you should instead read the doxygen output. This file has been tested
//! with Doxygen 1.3.8; to know how to create documentation from sources
//! and from this file, please read the docs/ReadMe.html file.
///////////////////////////////////////////////////////////////////////


// some TODOs:

//! \todo When user moves cursor on math area, create a tooltip with type of data 
//!          and miscellaneous info about the under-cursor element
//! \todo Add a plotter with ExportToPOV function
//! \todo Add a queue to mcElement for special messages like gui_RecalcSize or CheckArray;
//!  in this way, tasks are accomplished only when all changes have been made (no
//!  multiple gui_RecalcSize inside one function).


// ---------------------------------------------------------------------------------
//! \page input_flow_chart Input flow chart
//!
//! When user types a key, its useful for the programmer to have a 
//! good idea about what code is executed. For more info on each function, 
//! see previous sections.
//! 
//! <center>
//! \image html input-flow-chart.gif
//! </center>


// ---------------------------------------------------------------------------------
//! \mainpage Main page
//! 
//! <center>
//! \image html MathCore_title.jpg
//! <h1>Developer's guide</h1>
//! </center>
//! 
//! <center><P>Copyright &copy; 2003-2004 by 
//! <A HREF="mailto:frm@users.sourceforge.net">Francesco Montorsi</A></P></center>
//! 
//! <BR><P>This page and the following linked here have been written by hand; 
//! all the other pages were extracted from the sources' comments by Doxygen.
//! You can find links to these pages also in &quot;
//! <A HREF="pages.html">Related Pages</A>&quot; section.</P>
//! 
//! <P>To get the most updated version of this documentation, you have to 
//! download the CVS module of MathCore (you can find instructions to do this 
//! <A HREF="http://mathstudio.sourceforge.net/web/doc.shtml">here</A>) 
//! and then refer to the .\\docs\\ReadMe.html file.</P>
//! 
//! <P><STRONG>Note:</STRONG> usually, not all the documents below (those 
//! written manually) are completely up-to-date; ask to the admin or to the 
//! mailing list if you want the detailed list of currently updated documents.</P>
//! <BR><BR><BR>
//! 
//! \section sec1 Pages written by hand:
//! <TABLE ALIGN="center" WIDTH="90%" CELLSPACING="2" CELLPADDING="2" BORDER="0">
//! <TR>
//!     <TD WIDTH="50%" CLASS="md"><P>&nbsp;&nbsp;&nbsp;<A HREF="../../../index.html">
//!     MathStudio-project documentation main page</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>Allow access to all the project documentation<br>
//!     (only if this is the complete CVS module &quot;all&quot;)</EM></DIV></TD>
//! </TR>
//! <TR>
//!     <TD CLASS="md"><P>&nbsp;&nbsp;&nbsp;<A HREF="tutorial.html">
//!     Tutorial</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>The most important document which also guides 
//!     the reader through the following links.</EM></DIV></TD>
//! </TR>
//! <TR>
//!     <TD CLASS="md"><P>&nbsp;&nbsp;&nbsp;<A HREF="introduction.html">
//!     Introduction</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>Some basic info on...</EM></DIV></TD>
//! </TR>
//! <TR>
//!     <TD CLASS="md"><P>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
//!     <A HREF="Introduction.html#general_intro">MathCore</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
//!     ...what is MathCore</EM></DIV></TD>
//! </TR>
//! <TR>
//!     <TD CLASS="md"><P>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
//!     <A HREF="Introduction.html#input_intro">MathCore input system</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
//!     ...the input system of MathCore</EM></DIV></TD>
//! </TR>
//! <TR>
//!     <TD CLASS="md"><P>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
//!     <A HREF="Introduction.html#solver_intro">MathCore engine</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
//!     ...the math engine of this library</EM></DIV></TD>
//! </TR>
//! <TR>
//!     <TD CLASS="md"><P>&nbsp;&nbsp;&nbsp;<A HREF="mc_design.html">
//!     MathCore design</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>Useful graphs and examples</EM></DIV></TD>
//! </TR>
//! <TR>
//!     <TD CLASS="md"><P>&nbsp;&nbsp;&nbsp;<A HREF="input_flow_chart.html">
//!     Input flow-chart</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>Another useful graph</EM></DIV></TD>
//! </TR>
//! <TR>
//!     <TD CLASS="md"><P>&nbsp;&nbsp;&nbsp;<A HREF="math.html">
//!     MathCore formulas and algorithm used</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>The math behind MathCore</EM></DIV></TD>
//! </TR>
//! <TR>
//!     <TD CLASS="md"><P>&nbsp;&nbsp;&nbsp;<A HREF="authors.html">
//!     Authors</A></P></TD>
//!     <TD><DIV CLASS="groupText"><EM>The programmers who works or 
//!     worked on MathCore</EM></DIV></TD>
//! </TR>
//! </TABLE>


/*! \page authors Authors

<P>The following is the list of the programmers who works or worked on MathStudio, in
chronological order:</P>

<TABLE ALIGN="center" WIDTH="90%" CELLSPACING="2" CELLPADDING="2" BORDER="0">
<TR>
    <TD WIDTH=80%><DIV CLASS="ah"><P>- Francesco Montorsi -</P></DIV></TD>
    <TD><DIV ALIGN="right">[frm@users.sourceforge.net]</DIV></TD>
</TR>
<TR>
    <TD COLSPAN=2><P>The administrator of the project; he designed and developed the main 
classes. Send him an email for any question or suggestion about the project.</P></TD>    
</TR>
</TABLE>


<TABLE ALIGN="center" WIDTH="90%" CELLSPACING="2" CELLPADDING="0" BORDER="0">
<TR>
    <TD WIDTH=80%><DIV CLASS="ah"><P>- Omri Barel -</P></DIV></TD>
    <!-- <TD><DIV ALIGN="right"></DIV></TD> -->
</TR>
<TR>
    <TD WIDTH=80%><DIV CLASS="ah"><P>- Raphael Knaus -</P></DIV></TD>
    <!-- <TD><DIV ALIGN="right"></DIV></TD> -->
</TR>
<TR>
    <TD COLSPAN=2><P>Thanks to both for the help given while designing MathStudio
input system.</P></TD>
</TR>
</TABLE>


<TABLE ALIGN="center" WIDTH="90%" CELLSPACING="2" CELLPADDING="2" BORDER="0">
<TR>
    <TD WIDTH=80%><DIV CLASS="ah"><P>- Luciano Cattani -</P></DIV></TD>
    <TD><DIV ALIGN="right">[luccat@users.sourceforge.net]</DIV></TD>
</TR>
<TR>
    <TD COLSPAN=2><P>Thanks to Luciano Cattani who created the port for Linux and GCC,
	wrote the makefiles and corrected a lot of bugs in the Linux version.</P></TD>
</TR>
</TABLE>
*/








/*! \page introduction Introduction to MathCore

\section general_intro MathCore general introduction

<DIV STYLE="margin-left: 3%"><P>MathCore is a library aimed to provide easy-to-use and
powerful math functions to a graphical program. It is different from other libraries 
of this type both for two main things:
\li because of the way the user inputs the expression (all the classes
of MathCore implement a set of functions, derived by mcElementGUI, which provide a lot of functions for an easy and fast user input, which results really intuitive);
\li because of its data processing system: flexible and highly customizable, it can show to the user the operations and the intermediate steps done toward the final solution. The intermediate
steps towards resolution of the data (an expression, an equation or an inequality), are
almost always completely absent from the output of other math engines: they work
as black-boxes and the output is the only thing the user can see; MathCore is different.
The user can access also basic math functions (divideby, add, subtract, multiplyby...)
and thus he can manipulate the data with the help of the computer in the way he likes.

<BR>This document explains how the library is structured. 
This document is thus technical and just for programmers.<BR>
The following are some of the actual features of MathCore library:</P></DIV>

<TABLE ALIGN="center" WIDTH="80%" CELLSPACING="2" CELLPADDING="2" BORDER="0">
<TR>
	<TD CLASS="md">Development Status:</TD>
	<TD><P>Beta</P></TD>
</TR>
<TR>
	<TD CLASS="md">Intended Audience:</TD>
	<TD><P>Education, End Users/Desktop, Science/Research</P></TD>
</TR>
<TR>
	<TD CLASS="md">License:</TD>
	<TD><P>GNU General Public License (LGPL)</P></TD>
</TR>
<TR>
	<TD CLASS="md">Natural Language:</TD>
	<TD><P>English, Italian</P></TD>
</TR>
<TR>
	<TD CLASS="md">Operating Systems:</TD>
	<TD><P>Win32, Unix-like (using X11 or GTK), MacOS</P></TD>
</TR>
<TR>
	<TD CLASS="md">Programming Language:</TD>
	<TD><P>C++</P></TD>
</TR>
<TR>
	<TD CLASS="md">Topic:</TD>
	<TD><P>Education, Mathematics, Physics, Visualization, Text Editors</P></TD>
</TR>
<TR>
	<TD CLASS="md">Compilers used:</TD>
	<TD><P>On Win32: MSVC++, BorlandC++, MinGW, Cygwin<BR>On Linux/Unix/MacOS: GCC</P></TD>
</TR>
<TR>
	<TD CLASS="md">External libraries/components:</TD>
	<TD><P>wxXml2 (which uses iconv, libxml2),<BR>
		wxWidgets (which uses jpeg, tiff, png, expat...),<BR>
		wxScript (which uses CINT, lua, tolua, underc),<BR>
		mapm </P></TD>
</TR></TABLE>
 
<DIV STYLE="margin-left: 3%">
<P>The general features that will be implemented in next releases are listed in the version 
<A HREF="http://mathstudio.sourceforge.net/Web/log.shtml">log file</A> on MathStudio website.<BR>

The technical new features (you must know something about the program's structure)
are listed in the TODO file linked in &quot;<A HREF="pages.html">Related Pages</A>&quot; section.
<BR>The following sections of this document (specially the 
&quot;<A HREF="mc_design.html">MathCore design</A>&quot; page)
reference to the source code that you should have downloaded with this file. 
To understand how MathCore works, it's very useful to print the header files of 
the various classes: all the code is heavily commented and easy to read. 
In particular, to know more about the codes and defines used by MathCore functions, 
I suggest you to print the &quot;Element.h&quot; header file.

<BR>If you are not a member of the project but you
are interested in it and you would like to contribute in any way (as a
programmer, help editor or anything else useful...) you are welcome, just contact
the administrator writing an email to <a href="mailto:frm@users.sourceforge.net">frm@users.sourceforge.net</a>.

<BR>This is one of the first versions of this help system:
many things may need to be better explained and many things may be missing; if
you don't understand something of the issues explained here, please don't
hesitate and contact the MathCore administrator
writing an email to <a href="mailto:fr.m@iol.it">frm@users.sourceforge.net</a>.</P>
</DIV>
<BR>
<BR>
<BR>




\section input_intro MathCore input system introduction

<DIV STYLE="margin-left: 3%"><P>One of the main parts of MathCore
is the data input system; many efforts were done to make it really intuitive
and easy, take a look at the following example:<BR><BR></P></DIV>


\image html example1.gif


<DIV STYLE="margin-left: 3%">
<P>Notice that the styles which compose the output 
(the fonts, the colors) can be configured by the user. The appearance of the output can 
be personalized in a lot of things; this
program, in fact, can be used not only to solve expressions and equations but also to 
easily create &quot;snapshots&quot; of mathematical data to insert as bitmaps in programs 
which don't support the entering of such expressions (many word processor, for example).</P>

<P>A simple GUI input is one of the keypoint of MathCore structure: this feature, together
with a flexible (that is, completely user-accessible) math engine will make much easier,
I hope, the spread of math editing also for all those simple users which never heard about
such programs and do not have time to read a 1500-pages manual just to type in a simple formula...</P>
</DIV>

<BR>
<BR>
<BR>




\section solver_intro MathCore math engine introduction

<DIV STYLE="margin-left: 3%">
<P>MathCore is a symbolic manipulation library.
A symbolic manipulator is much more different from the usual calculators that students
use at school. A scientific calculator is used just to get the result of long operations
which involve a big number of digits; these high-precision systems are called numeric
analysis tools: they can handle any type of math operation and function computation but
they cannot work with unknowns and parameters. Many of them (like the powerful GNU
Scientific Library, GSL) implement some functions that, using algebraic algorithms,
give the user the solutions of a generic equation/inequality; anyway, they never show
any of the intermediate steps being these usually very very complex...<BR>

A symbolic system, instead, works with unknowns and parameters using all the math rules
that everyone learns at school; usually, also the algorithms of a symbolic system are
very complex (generally, more than those implied with numeric analisys !) and thus, 
also many CASes (Computer Algebra System) do not show any intermediate step, just the
solution; the high complexity of these programs usually implies also a big price !<BR>
A good example of open-source general (many little libraries CAS-like are aimed to solve
some specific math problems related, for example, to matrices, tensors...) CAS, even if quite difficult to use, is <a href="maxima.sourceforge.net">Maxima</a>.<BR>

MathCore is aimed to give a good support for all basic math operations, that is, simplification, expansion, resolution of systems involving sets of 1st, 2nd and 3rd degree equations and inequalities. In particular, the system is aimed to be able to show to the user all the procedure, step-by-step, used to reach the solutions. The creation of such a system, that 
emulates the human way of equation-solving, is much complex and requires much work, specially
if the programmers working in this open-source project (like me) never coded a CAS before...
please don't be too severe when trying the program :-).<BR>

By now, the library, regarding math engine, is structured in this way: all elements (fractions,
brackets, monomials, polynomials...) expose an interface that allows the manipulation at all
levels. Besides, a set of solvers (derived from mcSolver basic interface) implements the
algorithms that, using those basic functions, solve the data, in some ways specific for
each type of input (that is, using an algorithm when solving polynomial equations and another
one when solving trigonometric ones), producing the step-by-step resolution procedure.</P>
</DIV>


<HR>
	
<center>
<P>MathCore project makes use of many libraries. See the table in the &qout;MathCore general introduction&qout; above for the complete list.</P><BR>

<TABLE ALIGN="center" BORDER=0>
<TR>
<TD>\image html LibXML-logo.gif 
</TD>
<TD>\image html builtwithwx.gif 
</TD>
</TR>
</TABLE>

<BR>
<P>MathStudio is a project hosted by <A HREF="http://sourceforge.net/">SourceForge</A></P>

\image html SF-logo.gif

<P>All trademarks are the property of their respective owners.</P></DIV>

*/






/*! \page mc_design MathCore design issues

<P>
<A HREF="#memory">Memory structure of math data</A><BR>
<A HREF="#global">Global design choices</A>
</P>

<hr><H2 ID="memory">Memory structure of math data</H2>

<!-- <P>MathCore is a library written in C++; it uses many wxWidgets
classes (if you read the sources, all the objects whose names begin with 'wx'
belong to the wxWidgets library).</P> -->

<P>Every math expression can be considered a tree where the
leaves are the smallest units (numbers, parameters, unknowns) and the other
nodes are the containers of the leaves (brackets, fractions, radicals).
This tree-structure is well-suited for a math library because it allows the
creation of complex expressions using <STRONG>recursion</STRONG> (an example
is shown later). Other approaches (like reverse polish notation) has already
been tried, but they revealed to be too complex to handle or too complex to
edit by math functions.</P>

<P>The following picture sums up the relations
among main MathCore classes:</P>

<center>
\image html inherit__graph__0.png

</DIV>

<P>For each class of this graph, there are four other classes deriving from
mcElementData, mcElementGUI, mcElementMath and mcElementIO; they contain the data,
the GUI, the math and the input-output stuff of the element. These classes
are referred to as "helper classes": in fact, they are like
subsections of the same element. This division is
required especially to keep math and GUI functions separed 
(in a first approach, the mcElement-derived classes contained all
GUI, math & IO stuff, but that approach revealed to be really buggy and
difficult to handle).<BR>
As you can see in the <A HREF="inherits.html">main inheritance graph</A>, 
some elements, like mcAddOp and mcSubOp, doesn't implement all 
the helper classes. In fact, this is not required, because mcAddOp and mcSubOp 
uses all the GUI, Math and IO stuff implemented in mcOperator.<BR>
For more info about this structure, you can read the <A HREF="tutorial.html">tutorial</A>.
</P>

<P>To understand how mathematical data is stored in MathCore classes, I prepared
a graph I hope will help you.</P><br>

<TABLE width="90%" border="0">
<tr>
	<td>\image html example2-1.gif</td>
	<td><p>On the left there is the graphical representation of the structure showed below.</p></td>
</tr>
</TABLE>

<center>
\image html example2-2.gif
</DIV>

<P>If this graph did not help you, I'll show you the output of the 'View.data_Debug Info'
menu command, showed by an application which uses MathCore, when compiled
in debug mode (of course, you can use it to get more examples):</P>


<TABLE ALIGN="center" CELLSPACING="20" CELLPADDING="20" BORDER="1">
<TR><TD>

<center>
\image html example3.gif

</DIV>


</TR></TD></TABLE>


<BR><P>As you can see a lot of work is done by mcPolynomial and mcMonomial classes; 
in fact mcElementArray (the base class of mcPolynomial and mcMonomial) 
plays a key role in MathCore and it is the most complex class.</P>



<hr><H2 ID="global">Global design choices</H2>

<P ID="global">MathCore is divided in some main parts; the following graph shows their relations 
with other software and hardware components of the machine where it runs:</P>

<center>
\image html ms-structure.gif

</DIV>

<P>As you can see, wxWidgets
framework cares about the graphical output while CINT (planned to be used from
version 0.8) and LibXML2
is used in the input system. To get more info about these libraries, please go
to <a href="http://www.wxWidgets.org/">http://www.wxWidgets.org</a>, <a
href="http://root.cern.ch/root/Cint.html">http://root.cern.ch/root/Cint.html</a>,
<a href="http://xmlsoft.org/">http://xmlsoft.org/</a>.</P>

<P>During the development of MathCore, many design choices have been done:</P>

<UL>
<LI><P><STRONG>Why the math, IO, GUI subsystems are not kept as three separate
libraries, each one specialized in its particular task ?</STRONG></P>
<P>The three &quot;subsections&quot; of MathCore are all packed in a single library
because, for example, it's very easier to create a flexible math library
if we can create the math tree just sendind to the library a string representation
of an inlined expression (that is, something like &quot;4x+5^x-sin(x)&quot;)...
in short, all the subsystem cooperates and make reciprocal tasks easier.
</LI>
<BR>

<LI><P><STRONG>Why the sign operators (+ and -) are outside monomials ?</STRONG></P>
<P>Keeping the mcAddOp and mcSubOp OUTSIDE mcMonomials, we can make very very
easier mcMonomial GUI input system and, in addition, we can make the most
of the mcElementArray class, which is designed as an array containing
elements interleaved by operators (and if we keep the sign operators _inside_ monomials, 
we would not take advatange of this for mcPolynomials, which would contain only
mcMonomials...).
For math aspects, then it's true: the sign should be stored inside the monomials
to make everything easier. To make a little sample: when mcPolynomials adds
two mcMonomials then it should always check its sign to update accordingly the
operator </P>
</LI>
<BR>

<LI><P><STRONG>Why do we use polynomials containing monomials and not 
directly polynomials containing elements ?</STRONG></P>
<P>Polynomials containing directly the various elements like mcFractions, mcNumbers... 
was the first approach but, even if this solution makes input system easier to
code, it was a real trouble for math system.</P>
</LI>
<BR>

<LI><P><STRONG>Why mcMathCore contains a lot of functions and variables which could be
kept inside mcElementGUI as static elements ?</STRONG></P>
<P>mcMathCore is planned to be only an allocator and the general mcElement class handler.
Every other variable and function which can be placed as STATIC in mcElementGUI, 
mcElementMath, mcElementIO will be placed there.</P>
</LI>
</UL>

*/













/*! \page tutorial MathCore tutorial


<P>The aim of this document is to teach to MathCore new programmers, to the users 
of the library and to curious people, the basic structure and the main design of 
the entire module.<BR>
After you read this tutorial, you should be able to: navigate the code (and the
documentation) to quickly find what you need; manipulate the math data created
by MathCore; use MathCore commands and, eventually, extend them.</P>

\note Sorry for my often erroneous english, but this is not my natural language :-)

<BR>
<P>
<A HREF="#prereq">Prerequisites</A><BR>
<A HREF="#conventions">Conventions</A><BR>
<A HREF="#physicstructure">Physical structure of the library</A><BR>
<A HREF="#structure">The library structure</A><BR>
<A HREF="#overview">Overview of main %mcElementGUI, %mcElementMath and %mcElementIO functions</A><BR>
<A HREF="#end">Further readings</A><BR>
</P>


<BR><HR><H2 ID="prereq">Prerequisites</H2>
<P>To understand this tutorial, you must meet two types of prerequisites: 
object-oriented programming in C++ and some basics about the libraries used
by MathCore.</P>

<P>The C++ prerequisites consist in the concepts of:
- class
- inheritance
- virtual function
- polymorfism
</P>

<P>About the libraries used, mapm, lua, tolua, underc, cint, iconv, libxml2 and especially <STRONG>wxWidgets</STRONG>, you should know only two main things:</P>
<OL>
<LI>what they are</LI>
<LI>why they are required</LI>
</OL>
<P>In fact, you will find that the usage of these libraries is limited to some
very specific areas of MathCore library (except for some useful classes like
wxString, wxPoint or wxDC, which are used almost everywhere).</P>


<BR><HR><H2 ID="conventions">Conventions</H2>
<P>In this document (and more generally, in the entire MathCore documentation)
I will use few conventions, which I list here:</P><OL>

<LI>with the word 'op' I mean 'operator': one of the mcOperator-derived classes
(mcAddOp, mcSubOp, mcMultOp or mcDivOp) which are used to represent binary operations
in the memory representation of math data .</LI>

<LI>All the classes begin with the 'mc' prefix and for member variables, a slightly
modified Hungarian notation is used.</LI>

<LI>All the comments in the header files are written using Doxygen syntax
(see the <A HREF="../ReadMe.html">Documentation readme</A>). Source files
contain few comments inside the functions only: you should browse the header
files if you want to read good docs.</LI>

<LI>With the word 'section', 'subsection' and 'helper class' I usually refer 
to one of the five parts
(Data, GUI, Math, IO, Elem) which compose a mcElement-derived class (which 
will be explained later).</LI>
</OL>


<BR><HR><H2 ID="physicstructure">Physical structure of the library</H2>
<P>The MathCore library is structured in various CPP and H files organized in the
<EM>src</EM> and <EM>include/mc</EM> folders; see the main
<A HREF="../../ReadMe.html">Readme</A> file for more info about the
module structure.</P>

<P>Anyway, the files can be distincted for their contents in three main
categories:</P>

<table cellspacing="0" cellpadding="10" align="center" border="1" width="100%">
	<tr>
		<td width="25%" align="center"><STRONG>Files containing mcElement-derived classes</STRONG><BR>&nbsp;</td>
		<td width="25%" align="center"><STRONG>Files containing mcSolver-derived classes</STRONG><BR>&nbsp;</td>
		<td width="25%" align="center"><STRONG>Files which implement special classes</STRONG><BR>(non mcElement-derived)&nbsp;</td>
		<td width="25%" align="center"><STRONG>Files which implement MathCore interfaces</STRONG><BR>(the most important classes for library users)&nbsp;</td>
	</tr>
	<tr>
		<td align="center">
		Bracket<BR>
		Element<BR>
		ElementArray<BR>
		EmptyBox<BR>
		ExpContainer<BR>
		ExpElement<BR>
		Fraction<BR>
		Function<BR>
		Text<BR>
		Monomial<BR>
		Number<BR>
		Operator<BR>
		OpTypes<BR>
		Polynomial<BR>
		Radical<BR>
		Symbol<BR>
		</td>

		<td align="center">
		Solver<BR>
		PolySolver<BR>
		BisectSolver<BR>
		</td>

		<td align="center">
		MathHelpers<BR>
		MathLine<BR>
		MathUtils<BR>
		MathMng<BR></td>


		<td align="center">
		MathCore<BR>
		MathSystem</td>
	</tr>
</table>

<P>Each file of MathCore follows some common rules: its name is the name of the
class which defines or implements (except for some files which contain more than
one definition/implementation) without the 'mc' suffix.</P>

<P>The files always begin with the LGPL disclaimer, then follows some file-specific
information (author, creation date...), the precompiled-header stuff and last the implementation. 
For mcElement-derived classes there are, in order,
the implementation of the element itself, its Data, GUI, Math and IO sections.</P>

<P>In the header files, after the disclaimer, an #ifdef - #endif block avoids more
than one inclusion of the same header file in the CPP files. Then, there are the
#includes of those headers required to correctly compile the class definition,
the #defines of the values used by the classes and last, the class definitions.</P>




<br><hr><H2 ID="structure">The library structure</H2>
<P>To understand the library structure, you must first know its aims:
you can read for this purpose &quot;<A HREF="introduction.html">MathCore general 
introduction</A>&quot;.<BR>

Once you know that this library provides facilities both for math and import/export
but also for graphical input/output, you should understand the need to divide the 
functions and variables associated with each one of these tasks.<BR><BR>

Functions belonging to mcElement-derived classes (that is, classes which usually wrap
mcElement features to hide MathCore internals) are organized in three great &quot;blocks&quot;: GUI, math, IO, which follow the same naming-rules of mcElement-derived sections.
Thus, for example, to use a GUI and an IO function of mcMathOrSystem, you just have to
write something like:<BR>

\code
mcMathOrSystem math;
math.gui_DrawSystemBmp(...);
math.io_ImportInlinedExpr("4x+5^(x-y)-sin(x_1)>=0");
\endcode

That is, mcMathOrSystem implements a lot of functions with the same names of mcElementGUI,
mcElementMath, mcElementIO functions, so it should be not too difficult to browse &amp;
use these MathCore special classes when you get used to the mcElement interface.

All mcElement-derived classes, instead, are composed up of five different classes,
tied together, with the same name except for the suffix, which can be one of:

<UL>
<LI><STRONG>Data</STRONG>: this section contains all the varaibles common to the five parts
of the mcElement-derived class and all the big structures which require a lot of memory, plus
the functions which manipulate them.</LI>

<LI><STRONG>GUI</STRONG>: here are grouped all the functions and variables used to render 
the elements on the screen, to move the cursor inside it by keyboard or mouse,
the functions to select it and the functions which handle keyboard inputs.
The most important variable which belongs to this section si the cursor position:
it is used in a lot of GUI functions.</LI>

<LI><STRONG>Math</STRONG>: all the functions which are used to perform math operations,
ranging from basics to advanced, are grouped in this section. In some special cases,
math functions are tied to GUI functions because they perform operations on selected
elements only (and selection is performed through GUI functions), but generally, 
this section should be completely autonomous.</LI>

<LI><STRONG>IO</STRONG>: this section contains all the import and export functions,
which allow MathCore to save and load, for example, XHTML or XML files containing
MathML (the web standard for math expressions) and inlined expressions (strings
like "4x+5^(x-y)-sin(x_1)>=0").</LI>

<LI><STRONG>[Elem]</STRONG>: this is the &quot;glue&quot; class. When this class
is allocated, it automatically creates all the other four classes and link them
together. So, a subsection of an element (e.g. mcFractionGUI, mcBracketIO, mcPolynomialMath...)
should *never* be created alone; instead an instance of the related mcElement 
(e.g. mcFraction, mcBracket, mcPolynomial...) should
be allocated, possibly through provided allocator functions (mcElement::NewElem, for
example).</LI>
</UL>

The mcElement-derived class which handles simple numbers, for example, is made up of
mcNumberData, mcNumberGUI, mcNumberMath, mcNumberIO and mcNumber (the glue classes do not
have any suffix, even if they are referred to as &quot;elem&quot;).<BR><BR>

To make easier the creation and the maintenance of all mcElement-derived classes (more than
one hundred classes if we count also all the different sections), some macros are widely
used (for further info on these ones, see Doxygen documentation):</P>

<TABLE width="95%" cellspacing="0" cellpadding="10" align="center" border="2">
	<tr>
		<td valign="middle" width="50%">
		<center><P><STRONG>#mcDEFINE_HELPER_CLASSES()</STRONG><BR>
		<STRONG>#mcDEFINE_FRIEND_CLASSES()</STRONG></P></DIV>
		</td>

		<td valign="middle" width="50%">
		<P>Two useful macros which define all the classes associated with a
		mcElement-derived class.</P>
		</td>
	</tr>
	<tr>
		<td valign="middle" width="50%">
		<center><P><STRONG>#mcDEFINE_ALLDATA_GETTERS()&nbsp;&nbsp;</STRONG><BR>
		<STRONG>#mcDEFINE_ALLGUI_GETTERS()&nbsp;&nbsp;</STRONG><BR>
		<STRONG>#mcDEFINE_ALLMATH_GETTERS()&nbsp;&nbsp;</STRONG><BR>
		<STRONG>#mcDEFINE_ALLIO_GETTERS()&nbsp;&nbsp;</STRONG><BR>
		<STRONG>#mcDEFINE_ALLELEM_GETTERS()&nbsp;&nbsp;</STRONG></P></DIV>
		</td>

		<td valign="middle" width="50%">
		<P>These macros are very important: they define the getters which are
		used in the various classes to access the other sections of the same element.
		The getters in total are 10: data(), gui(), math(), io(), hlp() and their
		const version (that is, functions with the same names returning \c const pointers).<BR>
		A simple example could be:</P>

		\code
		void mcMyElem::gui_MyFnc() {

			// using the math() getter, a pointer to mcMyElemMath
			// class is returned:
			math_MultiplyBy(...);

			// the same is for data(), io(), hlp() getters and their
			// const counterparts; obviously the gui() and gui()
			// getters are not defined in mcMyElemGUI...
			data_DeepCopy(...);
			io_GetMathML(...);
			hlp()->GetId();
		}
		\endcode

		</td>
	</tr>
</TABLE>

<P>In conclusion, a mcElement-derived class is really composed of five helper
classes, none of which can be created alone. In fact, the constructor of 
the glue classes (those derived directly by mcElement) creates immediately
also the other four classes, hiding this task to the programmer.<BR>
However, this does not happen always: mcMathCore allows the user of the
library, through the mcMathCore::Setup function to init only the GUI, the
MATH or the IO systems (or a combination of them) of the library to save
memory and processor time. If the sections which will be enabled are chosen
at run-time, then you will need to write safe code:
</P>

\code
mcNumber n(parent);
if (mcMathCore::isGUIEnabled()) n.gui_Draw(...);
if (mcMathCore::isMathEnabled()) n.math_MultiplyBy(...);
if (mcMathCore::isIOEnabled()) n.io_ImportPresentationMathML(...);
\endcode



<br><hr><H2 ID="overview">Overview of main mcElementGUI, mcElementMath and mcElementIO functions</H2>
<P>mcElement provides a lot of functions, a wide abstract interface, which could be
'confusing' for the user who wants to use it. There are not too many main functions,
but there are a lot of smaller methods which use them (before implementing a 'new'
function, search for similar ones: many times I created functions that I found to be
almost identic to the ready-made ones I coded some months before !). 
It's a Good Thing to have an
idea of the main functions for the three main sections of each mcElement: this
allows a faster search among the function lists (all the doxygen documentation is
indispensable anyway !) and a better comprehension of the library.

<br><h3>mcElementGUI</h3>
<ul>
<li>All mcElementGUI output functions use the mcStyles facilities to render strings.
A mcStyle class contains the mcTextSetting classes referring to all the types of
mcElement-derived class and thus they contain the info to render any type of element.
To each exponent level (in the expression a^b^c^d, the 'a' has exponent level zero,
the 'b' has exponent level one, the 'c' an exponent level equal to two...) is
associated a different mcStyle (each exponent level requires a smaller font of the
previous one) which allow a great rendering flexibility (different fonts &amp; colors
can be chosen for different exponent levels and for different element types).
mcElementGUI holds static pointers to a mcStyleArray which are usually setup by
mcMathCore::Setup.<BR>
Anyway, mcStyle-access functions include some powerful functions like mcElementHelpers::sgui_SelectStyle or mcElementHelpers::sgui_GetSizeOf; also other functions, like mcElementHelpers::sgui_GetSize are
strictly related to the current mcStyle selected.</li>

<li>mcElementHelpers::sgui_Draw is a simple but powerful function to render the elements
on a given DC at a given position; obviously, container elements (like mcPolynomial,
mcBracket...) when drawn, automatically call the gui_Draw() functions of their children.
This function is also useful to give to the user a feedback, highlighting those elements
placed below the mouse cursor.</li>

<li>Cursor-related functions are very important for mcElementGUI. To get the coords
of the current cursor position just use the mcElementHelpers::sgui_GetRelCursorPos and to 
move the cursor in response to a user keypress, use mcElementHelpers::sgui_MoveCursor or
mcElementHelpers::sgui_MoveCursorUsingPoint when moving it through the mouse.</li>

<li>User input is performed through the mcElementHelpers::sgui_Input function: this handles
not only correct inputs, but also invalid keypress (generating syntax errors) and
BACKSPACE and CANCEL keypress.</li>

</ul>



<br><br><h3>mcElementMath</h3>
<p>[THIS SECTION IS NOT COMPLETE AND WILL CHANGE IN FUTURE]</p>
<ul>
<li>CLASSIFICATION: since MathCore is designed to use different approaches for different types of problems,
the mcElement::math_GetMathType function is very important to (roughtly) classify the type
of data the engine must work on. See mcMathType for more info on the classification system
currently used.</li>

<li>COMPARISON: it's very important for a CAS to know if something is mathematically equal
to another one; it's also important to know if they are exactly identic (this check is
performed by mcElement::math_CompareWith) or if they are just <em>equivalent</em> through
some form of simplification (this check is performed by mcElement::math_hasSameContentOf).</li>

<li>EVALUATION: even if MathCore is a symbolic system, it still needs in some cases to
evaluate an expression and get a numeric value (which can be an approximated value if
the expression to evaluate involves periodic numbers or real constants, like PI); this
is the task of mcElement::math_Evaluate.</li>

<li>SEARCHING: recursive search are performed by the mcElement::math_Find; this is
important to get the coefficients of a polynomial equation, for example, or to
classify the data...</li>

<li>SIMPLIFICATION/EXPANSION: these are very powerful functions whose objectives are, in order,
make the data shorter and make the data longer. The 'lenght' of data is measured by the
mcElement::math_GetLenght function. [TOWRITE]</li>

<li>BASIC OPERATIONS: basic operations are handled by two sets of functions. The first
set, mcElement::math_Add, mcElement::math_MultiplyBy, mcElement::math_,
mcElement::math_DivideBy return a mcBasicOpRes enumeration structure which can be
a bit problematic to handle. The second set, mcElement::math_SimpleAdd, ..., 
is designed to make these functions easier (but they cannot be used always).
All these functions are the raw-level edit functions for high-level math functions.</li>
</ul>



<br><br><h3>mcElementIO</h3>
<ul>
<li>MathCore has a good support for MathML both for input and output: the
library user just have to load a document through the wxXml2 wrappers for
the libxml2 library and then pass the MathML presentation root node to 
mcElement::io_ImportPresentationMathML. To export just use the
mcElement::io_GetMathML.</li>

<li>Inlined expressions are those strings which are commonly used (and were
used also above in this tutorial) to encode math data (which usually involves
exponents, subscripts, special symbols...) with an ANSI sequence of inlined
chars. MathCore uses the convention to use the '^' symbol to introduce
exponents and the '_' symbol to introduce subscripts. This type of input and
specially output is quite fast, through the mcElement::io_GetInlinedExpr and
mcElement::io_ImportInlinedExpr functions.</li>
</ul>





<br><hr><H2 ID="mathsystem">Overview of mcMathOrSystem</H2>
<P>[TOWRITE]</P>




<br><hr><H2 ID="end">Further readings</H2>
<P>If you want some further info about MathCore design you should read the
&quot;<A HREF="mc_design.html">MathCore design</A>&quot; page which also explains
the memory structure of math data.<BR>
There are no many docs yet on MathCore internals mainly because I am a programmer
and I do not love to spend my time writing these documents :-), and also because 
the project is quite young (created the 9th September 2003).
Anyway, if you stil have something unclear, please contact me at 
<a href="mailto:frm@users.sourceforge.net">frm@users.sourceforge.net</a> and I will 
be happy to discuss your questions.
</P>

<br>
<div align="right">
<P>Thanks for tolerating my bad english...<BR>
The author, Francesco Montorsi</p>
</div>

*/



/*! \page math MathCore formulas and algorithms used


<P>This page provides a list of the algorithms used by MathCore to perform some
of its math tasks. The following are not intended to be rigorous proofs, since
I am not a math-teacher and the math involved is not too difficult.</P>
<BR><BR><BR>


\section The mcPolynomial::math_RaiseTo function
<P>The mcPolynomial::math_RaiseTo function implements the algorithm used by
MathCore to raise to the n-th power (where  <STRONG><EM>n</EM></STRONG> is an integer value) a generic
<STRONG><EM>m</EM></STRONG>-terms polynomial.</P>
<P>Which formula does it use ? Consider the classical Newton's binomial formula:</P>

<center>
\image html binomial.png
</DIV>

<P>This formula is the formal rule used to raise a binomial to a power <STRONG><EM>n</EM></STRONG>.</P>
<P>Since we want to have the explicit formula for a generic polynomial, we can think
<STRONG><EM>b</EM></STRONG> as being the sum of two other monomials <STRONG><EM>c</EM></STRONG> and <STRONG><EM>d</EM></STRONG>:</P>

<center>
\image html polynomialraise1.png
</DIV>

<P>Since <STRONG><EM>(c+d)^k</EM></STRONG> is another binomial, we can apply 
Newton's formula on it again using another iterator <STRONG><EM>j</EM></STRONG>, obtaining:</P>

<center>
\image html polynomialraise2.png
</DIV>

<P>If we try to substitute to <STRONG><EM>d</EM></STRONG> the sum of 
<STRONG><EM>e</EM></STRONG> and <STRONG><EM>f</EM></STRONG>, we get:</P>

<center>
\image html polynomialraise3.png
</DIV>

<P>Performing such type of substitution for all members of our polynomial, we
get:</P>

<center>
\image html polynomialraise4.png
</DIV>

<P>where <STRONG><EM>m</EM></STRONG> is the number of the monomials contained in the 
polynomial to raise and all the <STRONG><EM>k</EM></STRONG> are the iterators required.</P>


*/



