///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   EmptyBox.h
//! \brief  Defines the mcEmptyBoxData, mcEmptyBoxGUI, mcEmptyBoxMath,
//!         mcEmptyBoxIO, mcEmptyBox classes.
//!
//! See EmptyBox.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef EMPTYBOX_H
#define EMPTYBOX_H


// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "EmptyBox.h"
#endif

// required includes
#include "mc/Decoration.h"


mcDEFINE_HELPER_CLASSES(mcEmptyBox)


//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcEmptyBoxHelpers : public mcDecorationHelpers
{
public:
	mcDEFINE_REFERENCE_DATA(mcEmptyBox, mcET_EMPTYBOX);

public:
	mcEmptyBoxHelpers() { data_Init(); }
	virtual ~mcEmptyBoxHelpers() {}


#ifdef mcENABLE_DATA
public:

#ifdef __MCDEBUG__

	//! Returns the name of this class.
	wxString data_Debug(long flags) const		{ return wxT("mcEmptyBox\n"); }

#endif
#endif		// mcENABLE_DATA



#ifdef mcENABLE_GUI
public:				// customizable variables
		
	//! The width of an empty box in "character units".
	//! To make this value work for all the styles, mcEmptyBoxGUI uses this value  
	//! which is multiplied by GetCharWidth() to get the pixel width of an empty 
	//! box; in this way empty boxes have the right size in each style.
	static float sgui_fEmptyBoxCharWidth;

	//! See mcEmptyBox::gui_m_fEmptyBoxCharWidth.
	static float sgui_fEmptyBoxCharHeight;

	//! The brush used to draw the empty boxes.	
	static wxBrush *sgui_pEmptyBoxBrush;

	//! The key to use to create this element.
	static mcKey *sgui_pNewEmptyBox;


public:

	/////////////////////////////////////////////////////////////////////////////////
	//! \name mcDecoration overloaded functions
	//! Implements the empty boxes' behaviours. Even if the mcDecorationGUI
	//! class implements almost all mcElementGUI abstract functions, here
	//! we need to override some of them because they suppose two cursor
	//! positions for a decoration, while mcEmptyBox has one cursor position
	//! only... @{
	
	//! Always returns FALSE: empty boxes should be created by
	//! mcPolynomials only.
	bool gui_isBeginKey(const mcKey &key) const {
		if (key.MatchKey(*sgui_pNewEmptyBox))
			return TRUE;
		return FALSE;
	}
	
	//! Always returns FALSE: mcEmptyBox::gui_Input function handles all the keypresses
	//! returning mcIR_DELETE_this-> Empty boxes infact, should always be replaced
	//! by a new class.
	bool gui_isEndKey(const mcKey &key) const;
		
	int gui_Draw(wxDC &dc, int x, int y, long flags, const wxPoint &pt) const;
	mcInputRes gui_Input(const mcKey &key, mcElement *pnew);
	mcInsertRes gui_Insert(const mcElement &, mcElement *);
	mcMoveCursorRes gui_MoveCursor(mcMoveCursorFlag flag, long modifiers);
	int gui_MoveCursorUsingPoint(wxDC &dc, const wxPoint &p);
	int gui_GetRelCursorPos(wxDC &dc, wxPoint *pt) const;
	void gui_DoRecalcSize();

	void gui_SetCursorPos(const mcCursorPos &code) {}
	void gui_GetCursorPos(mcCursorPos &p) const
		{ p.gui_Push(mcCP_BEGINEND); }

	//@}

#endif		// mcENABLE_GUI



#ifdef mcENABLE_IO
public:

	bool io_isBeginTag(const wxXml2Node &tag) const {
		if (tag.GetName() == wxT("memptybox"))
			return TRUE;
		return FALSE;
	}

	//! Returns the string used to export mcEmptyBoxes in inlined expressions.
	//! The string is always the same because empty boxes do not store any
	//! info inside them... so this function can be static.
	static wxString io_GetInlinedExprTag() { return wxT("[~]"); }

	wxXml2Node io_GetMathML(bool bGetPresentation) const;
	wxString io_GetInlinedExpr() const;	

	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);	

#endif		// mcENABLE_IO
};


//! A simple box which warns the user that he left (or he hasn't still
//! typed) something important in the polynomial he is editing.
class mcEmptyBox : public mcDecoration 
{
	// this class doesn't have any specific variable; it's just a place
	// where all the empty boxes' behaviours are described by the
	// following functions...
	mcDEFINE_MAIN_CLASS(EmptyBox, mcDecoration);
	
public:

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_EMPTYBOX; }
};

#endif	// EMPTYBOX_H





