///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   GenericSolver.h
//! \brief  Defines the mcSimplifySolver and mcExpandSolver classes.
//!
//! See GenericSolver.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef GENERICSOLVER_H
#define GENERICSOLVER_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "GenericSolver.h"
#endif

// required includes
#include "mc/Solver.h"



//! Just uses the mcElement::math_Simplify command until it returns mcESR_DONE.
class mcSimplifySolver : public mcSolver
{
public:
	mcSimplifySolver() {
		m_strName = wxT("Simplifier solver");
		m_strDesc = wxT("Repeatedly applies the SIMPLIFY command to the data.");
		m_nType = mcST_SIMPLIFY;
	}

	virtual ~mcSimplifySolver() {}

	//! Just uses the mcSolver::math_DoCompleteSimplification to fill the step array.
	bool math_SolveLine(mcMathLine &p, const mcSymbolProperties *unk, mcSystemStepArray &);

	bool math_WorksOn(const mcMathOrSystem &m) const
		{ return TRUE; }
	bool math_isReady() const							{ return TRUE; }

	wxPanel *gui_GetOptionsPanel() const				{ return NULL; }
};


//! Just uses the mcElement::math_Expand command until it returns mcESR_DONE.
class mcExpandSolver : public mcSolver
{
public:
	mcExpandSolver() {
		m_strName = wxT("Expander solver");
		m_strDesc = wxT("Repeatedly applies the EXPAND command to the data.");
		m_nType = mcST_EXPAND;
	}

	virtual ~mcExpandSolver() {}

	//! Just uses the mcSolver::math_DoCompleteExpansion to fill the step array.
	bool math_SolveLine(mcMathLine &p, const mcSymbolProperties *unk, mcSystemStepArray &);

	bool math_WorksOn(const mcMathOrSystem &m) const
		{ return TRUE; }
	bool math_isReady() const							{ return TRUE; }

	wxPanel *gui_GetOptionsPanel() const				{ return NULL; }
};


#endif		// GENERICSOLVER_H
