///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   MultDivOp.h
//! \brief  Defines the mcMultDivOp class.
//!
//! See MultDivOp.cpp for more info.
///////////////////////////////////////////////////////////////////////



#ifndef MULTDIVOP_H
#define MULTDIVOP_H

// optimization for GCC compiler
#ifdef __GNUG__
#pragma interface "MultDivOp.h"
#endif

// required includes
#include "mc/Operator.h"


mcDEFINE_HELPER_CLASSES(mcMultDivOp)



//! A little class containing the DATA, MATH, GUI and IO sections
//! of this element (as instances and not pointers).
//! This is entirely defined & implemented through macros.
class mcMultDivOpHelpers : public mcOperatorHelpers
{
	mcDEFINE_REFERENCE_DATA(mcMultDivOp, mdata_nType);

#ifdef mcENABLE_GUI
public:					// customizable variables
	
	//! If TRUE, a colon (wxT(";")) is used instead of "/" 
	//! when rendering mcDivOps.
	static bool sgui_bUseColon;
#endif


public:

	mcMultDivOpHelpers() { data_Init(); }
	virtual ~mcMultDivOpHelpers() {}


#ifdef mcENABLE_DATA
public:		// DATA variables

	bool data_isMultOp() const		{ return data_GetType() == mcET_MULTOP; }
	bool data_isDivOp() const		{ return data_GetType() == mcET_DIVOP; }

#ifdef __MCDEBUG__
	wxString data_Debug(long flags) const { 
		return wxString(wxT("mcMultDivOp  ")) + (data_isMultOp() ? wxT("*\n") : wxT("/\n")); 
	}
#endif
#endif



#ifdef mcENABLE_GUI
public:

	bool gui_isBeginKey(const mcKey &) const;
	int gui_Draw(wxDC &hDC, int Offsetx, int Offsety, long flags, const wxPoint &) const;
	void gui_DoRecalcSize();

#endif		// mcENABLE_GUI



#ifdef mcENABLE_MATH
public:

	bool math_CanBeApplied(const mcElement &, const mcElement &) const;
	mcRealValue math_Evaluate(mcRealValue v1, mcRealValue v2) const { return v1*v2; }

	mcBasicOpRes math_ApplySimple(mcElement &m1, const mcElement &m2) const;
	mcBasicOpRes math_Apply(mcElement &, const mcElement &, mcElement *) const;

#endif		// mcENABLE_MATH



#ifdef mcENABLE_IO
public:

	wxXml2Node io_GetMathML(bool bGetPresentation) const;
	bool io_ImportPresentationMathML(wxXml2Node tag, wxString &pErr);

#endif		// mcENABLE_IO
};


//! The operator which controls multiplication/division operations.
class mcMultDivOp : public mcOperator
{
	mcDEFINE_MAIN_CLASS(MultDivOp, mcOperator);

public:

	//! With this constructor, you can build a mcMultDivOp
	//! working as * or a mcMultDivOp working as /.
	mcMultDivOp(mcElementType t);
	

	bool isMultOp() const		{ return hlp()->data_isMultOp(); }
	bool isDivOp() const		{ return hlp()->data_isDivOp(); }

	bool data_isValidContainerFor(mcElementType t) const
		{ return t == mcET_MULTOP || t == mcET_DIVOP; }
};

#endif		// MULTDIVOP_H

