///////////////////////////////////////////////////////////////////////
// MathCore = a WYSIWYG equation editor + a powerful math engine     //
// Copyright (C) 2003 by Francesco Montorsi                          //
//                                                                   //
// This library is free software; you can redistribute it and/or     //
// modify it under the terms of the GNU Lesser General Public        //
// License as published by the Free Software Foundation; either      //
// version 2.1 of the License, or (at your option) any later         //
// version.                                                          //
//                                                                   //
// This library is distributed in the hope that it will be useful,   //
// but WITHOUT ANY WARRANTY; without even the implied warranty of    //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      //
// GNU Lesser General Public License for more details.               //
//                                                                   //
// You should have received a copy of the GNU Lesser General Public  //
// License along with this program; if not, write to the Free        //
// Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   //
// MA 02111-1307, USA.                                               //
//                                                                   //
// For any comment, suggestion or feature request, please contact    //
// the administrator of the project at frm@users.sourceforge.net     //
//                                                                   //
///////////////////////////////////////////////////////////////////////
//! \file   mc.h
//! \brief  MathCore main include file
//!
//! Includes all the MathCore headers. Besides, this file is included
//! by (almost?) all CPP files and thus this is the perfect header to
//! precompile.
///////////////////////////////////////////////////////////////////////


#ifndef MC_H
#define MC_H

// do not include the precompiled header of wxWidgets:
// it is included by mcprec.h

// pragma warnings are specific for MSVC++
#ifdef __VISUALC__

	// warning C4100: '' : unreferenced formal parameter
	#pragma warning (disable : 4100)

	// warning C4355: 'this' : used in base member initializer list
	#pragma warning (disable : 4355)

	// warning C4800: 'int' : forcing value to bool 'true' or 'false'
	#pragma warning (disable : 4800)

	// gmp.h(1599) : warning C4146: unary minus operator applied to unsigned type, result still unsigned
	#pragma warning (disable : 4146)

#endif


// first of all, include our configuration header.
// Makefiles which are not generated through the CONFIGURE script
// generated by the autoconf system, this file should have been
// generated (or copied) before start building MathCore
#include "mc/config.h"
#include "mc/Setup.h"


// define a special debug new() if using MSVC
/*#if defined(__MCDEBUG__) && defined(__VISUALC__)
	#include <crtdbg.h>
	#define mcDEBUG_NEW		new(_NORMAL_BLOCK, __FILE__, __LINE__)
#else
	#define mcDEBUG_NEW		new
#endif*/


// include wxXml2 extension for wxWidgets library
#include "wx/xml2.h" 		// libxml2 headers are required

// include wxScript extension for wxWidgets library:
// defining wxSCRIPT_NO_CINT/LUA/UNDERC macros we can avoid at
// compile-time the need of some headers/libs
#ifdef mcUSE_WXSCRIPT
	#include "wx/script.h" 
#endif

// include the main GMP header
#ifdef mcUSE_GMP
	#ifdef __VISUALC__
		#pragma warning(push, 2)
	#endif

	#include <gmp.h>		// The main include of the GNU Multiple Precision library

	#ifdef __VISUALC__
		#pragma warning(pop) 
	#endif
#endif


// include MathCore headers: THE ORDER IS IMPORTANT !!!
#include "mc/MathTypes.h"
#include "mc/MathUtils.h"
#include "mc/MathHelpers.h"
#include "mc/MathCore.h" 
#include "mc/Value.h" 


#include "mc/Element.h" 
#include "mc/Decoration.h" 
#include "mc/ElementArray.h" 
#include "mc/EmptyBox.h" 
#include "mc/Monomial.h" 
#include "mc/Polynomial.h" 

#include "mc/ExpContainer.h" 
#include "mc/ExpElement.h" 
#include "mc/Fraction.h" 
#include "mc/Bracket.h" 
 
#include "mc/MathMng.h" 
#include "mc/MathLine.h" 
#include "mc/MathSystem.h" 
#include "mc/MathAndSystem.h" 
#include "mc/MathOrSystem.h" 


#include "mc/Number.h" 
#include "mc/Operator.h" 
#include "mc/AddSubOp.h" 
#include "mc/MultDivOp.h" 
#include "mc/Parenthesis.h" 
#include "mc/Text.h"

#include "mc/Function.h" 
#include "mc/Radical.h" 
#include "mc/Symbol.h"  

#include "mc/Solver.h" 
#include "mc/GenericSolver.h" 
#include "mc/PolySolver.h" 
#include "mc/BisectSolver.h"

#endif		// MC_H
